"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
// @ts-ignore
const postcss_color_mod_function_1 = __importDefault(require("postcss-color-mod-function"));
const utils_1 = require("./utils");
const defaults_1 = require("./defaults");
const plugin = ({ prefix = '', colors = {}, base = defaults_1.BASE, sizes = defaults_1.SIZES, radius = defaults_1.RADIUS, }) => ({
    postcssPlugin: 'postcss-button-builder',
    OnceExit: (root, helpers) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const getDecl = (prop, value) => {
                const declaration = new helpers.Declaration({
                    prop: utils_1.kebabCase(prop),
                    value,
                });
                declaration.important = true;
                return declaration;
            };
            const getRule = (selector, attrs) => {
                const rule = new helpers.Rule({
                    selector,
                });
                rule.append(Object.keys(attrs).map((key) => getDecl(key, attrs[key])));
                return rule;
            };
            const basePrefix = `${prefix ? `.${prefix}-btn` : '.btn'}`;
            const baseRules = [
                getRule(basePrefix, base),
                getRule(`${basePrefix}:disabled, ${basePrefix}-disabled`, {
                    pointerEvents: 'none',
                    opacity: '0.5',
                }),
                getRule(`${basePrefix}-block`, {
                    display: 'block',
                    width: ' 100%',
                }),
            ];
            const appearancesRules = Object.keys(colors)
                .map((key) => (Object.assign({ color: key }, utils_1.getAppearances(colors[key]))))
                .flatMap((_a) => {
                var { color } = _a, appearances = __rest(_a, ["color"]);
                return Object.keys(appearances).map((appearance) => {
                    const baseSelector = `${basePrefix}-${appearance}-${color}`;
                    const { initial, hover } = appearances[appearance];
                    return [
                        getRule(baseSelector, Object.assign(Object.assign({}, initial), { outlineColor: `color-mod(${color} blackness(35%))` })),
                        getRule(`${baseSelector}:hover, ${baseSelector}:active`, hover),
                    ];
                });
            });
            const radiusRules = Object.keys(radius).map((r) => getRule(`${basePrefix}-radius-${r}`, { borderRadius: radius[r] }));
            const sizeRules = Object.keys(sizes).map((size) => getRule(`${basePrefix}-${size}`, sizes[size]));
            root.prepend(...baseRules, ...radiusRules, ...sizeRules, ...appearancesRules);
            const processor = helpers.postcss([postcss_color_mod_function_1.default()]);
            yield processor.process(root, { from: undefined });
        }
        catch (error) {
            throw root.error(error);
        }
    }),
});
module.exports = plugin;
